import sys, pprint
from colorboy import green
from vidl import config, __version__, dl, show_help, log

def main():
    config.verify_config()

    if len(sys.argv) <= 1 or '--help' in sys.argv or '-h' in sys.argv or sys.argv[1:] == ['help']:
        show_help()
    elif '--version' in sys.argv or sys.argv[1:] == ['-v'] or sys.argv[1:] == ['version']:
        log("Version", __version__)
    elif '--config-path' in sys.argv:
        log("Config path:", green(config.config_path))
    else:
        dl.main()

if __name__ == '__main__':
    main()



def user_md_parser(smart_md, md, video_info, url_info):
  # smart_md:
  #   Metadata object created by vidl. Metadata objects can have these properties:
  #     - title
  #     - artist
  #     - album
  #     - album_artist
  #     - track_number
  #     - track_count
  #     - year
  #     - comment
  #     - lyrics
  #     - composer
  # md:
  #   Same as "md", except this will never include smart metadata (artist/title
  #   parsed from title).
  # video_info:
  #   Object containing metadata from youtube-dl about the current video.
  # playlist_info:
  #   An object containing playlist metadata from youtube-dl.
  #   If the URL isn't a playlist, playlist_info is the same as video_info.
  #   If the URL is a playlist, it has an "entries" property with video_info objects.
  # callback: Callback function. Takes a metadata object as argument.

  # example:
  #   This example cheks if vidl has detected title and artist metadata. If the
  #   title includes "[NCS Release]", it will set the comment metadata to "NCS".
  # if 'title' in md and 'artist' in md:
  #   if '[NCS Release]' in md['title']:
  #     md['comment'] = 'NCS'

  return md



import mutagen
from mutagen.id3 import ID3, Encoding, TIT2, TPE1, TALB, TPE2, TRCK, TCON, TDRC, COMM, USLT, TCOM
from pprint import pformat
from vidl import log

def add_metadata(filename, md):
    try:
      tags = ID3(filename)
    except mutagen.id3.ID3NoHeaderError:
      file = mutagen.File(filename)
      file.add_tags()
      tags = file

    if 'title' in md:           tags["TIT2"] = TIT2(text=md['title'])
    if 'artist' in md:          tags["TPE1"] = TPE1(text=md['artist'])
    if 'album' in md:           tags["TALB"] = TALB(text=md['album'])
    if 'album_artist' in md:    tags["TPE2"] = TPE2(text=md['album_artist'])
    if 'track_number' in md:
        track_number = str(md['track_number'])
        if 'track_count' in md:
            track_number += '/'+str(md['track_count'])
        tags["TRCK"] = TRCK(encoding=3, text=track_number)
    if 'genre' in md:           tags["TCON"] = TCON(text=md['genre'])
    if 'year'   in md:          tags["TDRC"] = TDRC(text=md['year'])

    if 'comment'   in md:       tags["COMM"] = COMM(text=md['comment'], lang='eng')
    if 'lyrics'   in md:        tags["USLT"] = USLT(text=md['lyrics'])
    if 'composer'   in md:      tags["TCOM"] = TCOM(text=md['composer'])
    
    for key, value in md.items():
        whitespace = ' ' * (10 - len(key))
        log('  '+key+':'+whitespace+pformat(value))
        
    tags.save(filename)



# this file makes the folder a package
import pkg_resources, sys, pprint
from colorboy import cyan, green, red

package_name = __package__.split('.')[0] # used for getting config location and package version
package_author = 'Kasper Henningsen' # used for getting config location

# automatically set __version__
__version__ = pkg_resources.get_distribution(package_name).version

log_prefix = cyan('['+package_name+'] ')
class Log:
    def __call__(self, *args, **named_args):
        print(log_prefix+' '.join(args), **named_args)
    def pretty(self, *args, **named_args):
        pretty = pprint.PrettyPrinter(indent=4).pprint
        pretty(*args, **named_args)
    def error(self, *args, **named_args):
        print(log_prefix+red('Error: ')+' '.join(args), **named_args)
    def fatal(self, *args, **named_args):
        sys.exit(log_prefix+red('Error: ')+' '.join(args), **named_args)
log = Log()

def show_help():
    print(        '')
    print(green(  'Download Usage:'))
    print(        '    '+cyan(package_name)+' [format] [options] <URL>')
    print(        '')
    print(green(  'Download Options:'))
    print(cyan(   '    format             ')+'mp3, mp4, wav or m4a. Default mp3.')
    print(cyan(   '    --no-md            ')+'Don\'t add metadata to downloaded files.')
    print(cyan(   '    --no-smart-md      ')+'Don\'t extract artist and song name from title.')
    print(cyan(   '    --no-dl            ')+'Don\'t download anything. Usually used with -v')
    print(cyan(   '    -v, --verbose      ')+'Display all logs.')
    print(        '')
    print(green(  'General Options:'))
    print(cyan(   '    --version          ')+'Show version. '+green('vidl -v')+' and '+green('vidl version')+' works too.')
    print(cyan(   '    -h, --help         ')+'Show this help message. '+green('vidl help')+' works too.')
    print(cyan(   '    --config-path      ')+'Show the location of the configuration file.')
    print(cyan(   ''))
    print(green(  'Update:'))
    print(cyan(   '    pip3 install vidl --upgrade --upgrade-strategy eager'))
    print(        '')
